{%- extends "sphinx_rtd_theme/breadcrumbs.html" %}
{%- block breadcrumbs %}
{% endblock %}
{% block breadcrumbs_aside %}
<div style="float: right">
   <iframe src="https://ghbtns.com/github-btn.html?user=recognai&repo=rubrix&type=star&count=true&size=large" frameborder="0" scrolling="0" width="170" height="30" title="GitHub"></iframe>
</div>
{% endblock %}