/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'ignore': {
    width: 17,
    height: 18,
    viewBox: '0 0 17 18',
    data: '<defs><path pid="0" id="svgicon_ignore_a" d="M0 0h16.914v17.826H0z"/></defs><g _fill="none" fill-rule="evenodd"><path pid="1" d="M14.81 3.525l2.018-2.161L15.365 0l-2.076 2.224A7.937 7.937 0 008.914.913c-4.411 0-8 3.589-8 8a7.94 7.94 0 001.632 4.822L0 16.462l1.463 1.364 2.479-2.657a7.953 7.953 0 004.972 1.744c4.411 0 8-3.59 8-8a7.961 7.961 0 00-2.104-5.388M2.914 8.913c0-3.31 2.691-6 6-6 1.087 0 2.104.295 2.984.802L3.931 12.25a5.963 5.963 0 01-1.017-3.338m6 6A5.962 5.962 0 015.313 13.7l8.13-8.711a5.966 5.966 0 011.471 3.923c0 3.309-2.691 6-6 6" _fill="#000"/></g>'
  }
})