/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'predicted-ko': {
    width: 15,
    height: 18,
    viewBox: '0 0 15 18',
    data: '<path pid="0" d="M1.325 0h12.35C14.407 0 15 .592 15 1.322v11.474a1.32 1.32 0 01-1.322 1.322H4.237c-.881 0-1.875.65-2.227 1.453l-.61 1.393a.439.439 0 01-.368.242H0V1.322C0 .592.593 0 1.325 0zm7.01 7.115l1.483-1.482-.78-.78-1.482 1.482-1.482-1.482-.78.78 1.482 1.482-1.482 1.481.78.78 1.482-1.481 1.482 1.481.78-.78-1.482-1.481z" fill-rule="nonzero"/>'
  }
})