(window.webpackJsonp=window.webpackJsonp||[]).push([[41],{916:function(e,t,n){"use strict";n.r(t);var o={props:["date"],computed:{timeDifference:function(){return(new Date).getTimezoneOffset()}}},f=n(46),component=Object(f.a)(o,(function(){var e=this,t=e.$createElement;return(e._self._c||t)("span",[e._v(e._s(e._f("moment")(e._f("moment")(e._f("moment")(e.date,"utc"),"subtract",e.timeDifference+" minutes"),"from","now")))])}),[],!1,null,null,null);t.default=component.exports}}]);