/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'back': {
    width: 19,
    height: 15,
    viewBox: '0 0 19 15',
    data: '<path pid="0" d="M18.567 15H.834a.431.431 0 01-.433-.43c0-.237.194-.43.433-.43h17.3V5.245H1.724l5.214 3.689c.194.138.24.406.101.6a.435.435 0 01-.604.1L.18 5.206c-.215-.184-.262-.48 0-.7L6.436.08a.435.435 0 01.603.1.428.428 0 01-.1.6L1.841 4.384H18.52c.28 0 .481.226.481.481v9.705c0 .237-.194.43-.433.43z" _fill="#FFF" fill-rule="nonzero"/>'
  }
})