/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'cross': {
    width: 12,
    height: 12,
    viewBox: '0 0 12 12',
    data: '<path pid="0" d="M10.492.325L5.795 5.022 1.097.325a.643.643 0 10-.909.91L4.885 5.93.188 10.63a.643.643 0 00.91.909L5.794 6.84l4.697 4.698a.643.643 0 10.909-.91L6.704 5.932 11.4 1.234a.643.643 0 00-.91-.909z" _fill="#232425" fill-rule="nonzero"/>'
  }
})