/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'metrics': {
    width: 23,
    height: 18,
    viewBox: '0 0 23 18',
    data: '<path pid="0" d="M19.545.32v17.308h3.214V.321h-3.214zM6.723 2.888v14.741h3.214V2.888H6.723zm6.417 2.567v12.174h3.202V5.454H13.14zM.318 8.009v9.62h3.203v-9.62H.318z" _fill="#333346" fill-rule="nonzero"/>'
  }
})