/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'error': {
    width: 44,
    height: 44,
    viewBox: '0 0 44 44',
    data: '<g _fill="none" fill-rule="evenodd"><path pid="0" d="M24.335 24.335a2.372 2.372 0 01-2.366 2.365 2.372 2.372 0 01-2.365-2.365V12.508a2.372 2.372 0 012.365-2.365 2.372 2.372 0 012.366 2.365v11.827zm-2.366 9.46a2.367 2.367 0 010-4.73 2.366 2.366 0 010 4.73zM42.276 18.86L24.603 1.187a4.068 4.068 0 00-5.743 0L1.187 18.86a4.067 4.067 0 000 5.743L18.86 42.276a4.066 4.066 0 005.743 0l17.673-17.673a4.066 4.066 0 000-5.743z" _fill="#A5103A" mask="url(#mask-2)"/></g>'
  }
})