/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'filter': {
    width: 11,
    height: 12,
    viewBox: '0 0 11 12',
    data: '<path pid="0" d="M6.36 5.929c0-.185.093-.45.212-.599l3.424-4.268c.175-.218.088-.395-.201-.395h-8.59c-.286 0-.377.175-.2.395L4.427 5.33c.117.147.213.413.213.599v4.57c0 .46.381.834.859.834.475 0 .86-.372.86-.834v-4.57zm.687 0v4.57c0 .83-.692 1.501-1.547 1.501-.857 0-1.547-.672-1.547-1.501v-4.57a.545.545 0 00-.068-.19L.462 1.472C-.067.812.349 0 1.205 0h8.59c.86 0 1.27.815.743 1.471L7.115 5.74a.541.541 0 00-.068.189z" _fill="#000" fill-rule="nonzero"/>'
  }
})