/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'prev': {
    width: 20,
    height: 15,
    viewBox: '0 0 20 15',
    data: '<path pid="0" d="M0 7.466c0 .175.087.407.176.52l5 6.42c.234.29.612.325.884.029.243-.265.25-.78.024-1.068L2.071 8.222h17.304c.345 0 .625-.338.625-.755s-.28-.756-.625-.756H2.071l4.013-5.145C6.31 1.28 6.294.776 6.06.5c-.246-.293-.654-.267-.884.029l-5 6.42A.75.75 0 000 7.465z" _fill="#64B09F" fill-rule="nonzero"/>'
  }
})