/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'go-back': {
    width: 11,
    height: 18,
    viewBox: '0 0 11 18',
    data: '<path pid="0" d="M7.987.724L.981 7.714A1.537 1.537 0 00.98 9.89l7.007 6.994a1.617 1.617 0 002.289-2.284L4.472 8.803l5.802-5.792A1.616 1.616 0 107.987.724z" _fill="#FFF" fill-rule="evenodd"/>'
  }
})