/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'check3': {
    width: 11,
    height: 8,
    viewBox: '0 0 11 8',
    data: '<path pid="0" d="M10.224.197a.293.293 0 00-.21.089L3.566 6.7.993 4.11a.297.297 0 10-.424.418l2.78 2.8c.064.062.13.087.21.087a.287.287 0 00.208-.09l6.66-6.62a.292.292 0 00.004-.423.292.292 0 00-.207-.087v.001z" _fill="#686A6D" fill-rule="nonzero"/>'
  }
})