/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'list': {
    width: 15,
    height: 11,
    viewBox: '0 0 15 11',
    data: '<g _fill="#787878" fill-rule="evenodd"><path pid="0" d="M0 0h15v2H0zM0 3h15v2H0zM0 6h15v2H0zM0 9h15v2H0z"/></g>'
  }
})