/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'clock': {
    width: 13,
    height: 13,
    viewBox: '0 0 13 13',
    data: '<g _fill="#FFF" fill-rule="nonzero"><path pid="0" d="M10.128.217H2.872C1.313.217.092 1.84.092 3.914v5.172c0 2.074 1.221 3.697 2.78 3.697h7.256c1.56 0 2.78-1.624 2.78-3.697V3.914c0-2.073-1.22-3.697-2.78-3.697zm1.047 8.87c0 1.105-.563 1.963-1.047 1.963H2.872c-.484 0-1.047-.858-1.047-1.964V3.914c0-1.106.563-1.964 1.047-1.964h7.256c.484 0 1.047.858 1.047 1.964v5.172z"/><path pid="1" d="M7.747 5.676h-.612v-.528a.796.796 0 00-1.592 0V6.47c0 .44.356.796.796.796l.033-.003c.017.001.034.005.052.005h1.323a.796.796 0 000-1.592z"/></g>'
  }
})