/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'brand-isotipo': {
    width: 15,
    height: 21,
    viewBox: '0 0 15 21',
    data: '<defs><path pid="0" id="svgicon_brand-isotipo_a" d="M.827.296h13.584v19.998H.827z"/><path pid="1" id="svgicon_brand-isotipo_c" d="M0 .316h13.583v16.112H0z"/></defs><g _fill="none" fill-rule="evenodd"><g transform="translate(0 .587)"><mask id="svgicon_brand-isotipo_b" _fill="#fff"><use xlink:href="#svgicon_brand-isotipo_a"/></mask><path pid="2" d="M3.989 9.479h1.004c2.993 0 4.492-1.092 4.492-3.277 0-2.047-1.457-3.07-4.37-3.07H3.987v6.347zm3.935 2.293l6.487 8.522h-3.868l-5.984-8.186h-.57v8.186H.827V.296h3.704C7.3.295 9.3.791 10.53 1.784c1.357 1.105 2.036 2.565 2.036 4.377 0 1.417-.426 2.635-1.276 3.653-.85 1.02-1.972 1.671-3.365 1.957z" _fill="#FFF" mask="url(#svgicon_brand-isotipo_b)"/></g><g transform="translate(0 3.234)"><mask id="svgicon_brand-isotipo_d" _fill="#fff"><use xlink:href="#svgicon_brand-isotipo_c"/></mask><path pid="3" d="M8.658 2.336c0 2.185-1.497 3.277-4.492 3.277H3.162V.316H-.001v16.112h3.163V8.243h.57l5.984 8.185h3.867L7.096 7.907c1.394-.286 2.516-.938 3.366-1.958.85-1.018 1.275-2.235 1.275-3.652 0-.716-.11-1.375-.322-1.981H7.897c.505.496.76 1.169.76 2.02" _fill="#F2067A" mask="url(#svgicon_brand-isotipo_d)"/></g></g>'
  }
})