<!--
  - coding=utf-8
  - Copyright 2021-present, the Recognai S.L. team.
  -
  - Licensed under the Apache License, Version 2.0 (the "License");
  - you may not use this file except in compliance with the License.
  - You may obtain a copy of the License at
  -
  -     http://www.apache.org/licenses/LICENSE-2.0
  -
  - Unless required by applicable law or agreed to in writing, software
  - distributed under the License is distributed on an "AS IS" BASIS,
  - WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  - See the License for the specific language governing permissions and
  - limitations under the License.
  -->

<template>
  <div class="container">
    <h2 class="title">{{ title }} records ({{ dataset.results.total | formatNumber }})</h2>
  </div>
</template>

<script>
export default {
  props: {
    dataset: {
      default: () => {},
      type: Object,
    },
    title: {
      type: String,
      required: true,
    },
  },
};
</script>
<style lang="scss" scoped>
.container {
  @extend %container;
  padding-top: 1em;
  padding-bottom: 0;
  .fixed-header & {
    display: none;
  }
}
.title {
  font-weight: normal;
  @include font-size(16px);
  color: $font-secondary-dark;
  white-space: nowrap;
  display: flex;
  margin: 0.5em 0;
  align-items: center;
}
</style>
