/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'annotate-view': {
    width: 24,
    height: 24,
    viewBox: '0 0 24 24',
    data: '<path pid="0" d="M22.592 21.693c.464 0 .84.389.84.869s-.376.87-.84.87H1.33c-.464 0-.84-.39-.84-.87s.376-.87.84-.87h21.262zm-.01-15.767L9.263 19.723l-5.192.55.531-5.38L16.183 2.898l1.738-1.8c.781-.81 2.143-.81 2.924 0l1.738 1.8c.389.4.607.946.606 1.514 0 .572-.216 1.11-.606 1.515zM5.95 18.326l2.548-.27L19.064 7.111l-2.287-2.37L6.21 15.686l-.26 2.64zM19.383 2.208a.38.38 0 00-.275.118L17.964 3.51l2.288 2.37 1.143-1.184a.414.414 0 000-.57l-1.737-1.8a.38.38 0 00-.275-.118z" _fill="#333346" fill-rule="nonzero"/>'
  }
})