/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'predicted-ok': {
    width: 15,
    height: 18,
    viewBox: '0 0 15 18',
    data: '<path pid="0" d="M13.675 0C14.407 0 15 .62 15 1.383V18h-1.032a.438.438 0 01-.368-.254l-.61-1.456c-.352-.84-1.346-1.52-2.227-1.52H1.322C.592 14.77 0 14.152 0 13.385V1.383C0 .619.593 0 1.325 0h12.35zm-2.908 4.197a.655.655 0 00-.99.06L6.596 8.186 5.2 6.623a.655.655 0 00-.99-.007.832.832 0 00-.007 1.094l1.92 2.148a.67.67 0 00.498.23h.018a.673.673 0 00.504-.258l3.68-4.541a.83.83 0 00-.055-1.092z" fill-rule="evenodd"/>'
  }
})