/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'copy': {
    width: 11,
    height: 13,
    viewBox: '0 0 11 13',
    data: '<g _fill="#0508D9" fill-rule="nonzero"><path pid="0" d="M9.962.092H2.52a.627.627 0 00-.627.625V10.2a.627.627 0 00.627.625h7.442c.345 0 .625-.28.625-.625V.717a.625.625 0 00-.625-.625zm.075 10.108a.077.077 0 01-.075.075H2.52a.077.077 0 01-.077-.075V.717A.077.077 0 012.52.64h7.442a.077.077 0 01.075.076V10.2z"/><path pid="1" d="M8.534 11.82H1.063c-.026 0-.06-.035-.06-.088V2.28a.275.275 0 00-.55 0v9.449a.625.625 0 00.61.637h7.471a.275.275 0 100-.55v.004z"/></g>'
  }
})