/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'external-link': {
    width: 10,
    height: 10,
    viewBox: '0 0 10 10',
    data: '<g _fill="#F48E5F" fill-rule="nonzero"><path pid="0" d="M5.85 0v.6h3.126L4.788 4.788l.424.424L9.4 1.024V4.15h.6V0h-.6z"/><path pid="1" d="M9.4 7.95c-.001.8-.65 1.449-1.45 1.45h-5.9A1.452 1.452 0 01.6 7.95v-5.9C.6 1.25 1.25.6 2.05.6H4.2V0H2.05A2.053 2.053 0 000 2.05v5.9A2.053 2.053 0 002.05 10h5.9A2.053 2.053 0 0010 7.95V5.8h-.6v2.15z"/></g>'
  }
})