/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'arrow-bottom': {
    width: 17,
    height: 24,
    viewBox: '0 0 17 24',
    data: '<path pid="0" d="M8 21.45V1.028C8 .86 8.3.75 8.75.75s.75.111.75.278V21.45l6.225-6.225c.3-.3.75-.3 1.05 0 .3.3.3.75 0 1.05l-7.5 7.5c-.3.3-.75.3-1.05 0l-7.5-7.5A.81.81 0 01.5 15.75a.68.68 0 01.225-.525c.3-.3.75-.3 1.05 0L8 21.45z" _fill="#000" fill-rule="nonzero"/>'
  }
})