/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'progress': {
    width: 25,
    height: 25,
    viewBox: '0 0 25 25',
    data: '<g _fill="#333346" fill-rule="nonzero"><circle pid="0" cx="1.563" cy="12.5" r="1.563"/><circle pid="1" transform="rotate(-60 3.124 7.033)" cx="3.124" cy="7.033" r="1.563"/><circle pid="2" transform="rotate(-30 7.03 3.128)" cx="7.03" cy="3.128" r="1.563"/><circle pid="3" transform="rotate(-60 7.03 21.872)" cx="7.03" cy="21.872" r="1.563"/><circle pid="4" transform="rotate(-30 3.128 17.967)" cx="3.128" cy="17.967" r="1.563"/><path pid="5" d="M12.5 0v3.125a9.375 9.375 0 010 18.75V25C19.404 25 25 19.404 25 12.5S19.404 0 12.5 0z"/></g>'
  }
})