/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'double-chev': {
    width: 11,
    height: 7,
    viewBox: '0 0 11 7',
    data: '<g _fill="#686A6D" fill-rule="nonzero"><path pid="0" d="M7 6.767L7.25 7l3.5-3.267L11 3.5l-.25-.233L7.25 0 7 .233 10.5 3.5zM4 6.767L3.75 7 .25 3.733 0 3.5l.25-.233L3.75 0 4 .233.5 3.5z"/></g>'
  }
})