/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'zoomin': {
    width: 10,
    height: 10,
    viewBox: '0 0 10 10',
    data: '<g _fill="#000" fill-rule="nonzero"><path pid="0" d="M9.583 0H6.417a.417.417 0 100 .835h2.748v2.748a.417.417 0 10.835 0V.417A.417.417 0 009.583 0zM3.583 9.165H.835V6.417a.417.417 0 10-.835 0v3.166c0 .23.187.417.417.417h3.166a.417.417 0 100-.835z"/><path pid="1" d="M9.892.108a.369.369 0 00-.521 0L6.108 3.371a.369.369 0 00.521.521L9.892.629a.369.369 0 000-.521zM3.892 6.108a.369.369 0 00-.521 0L.108 9.371a.369.369 0 10.521.521l3.263-3.263a.369.369 0 000-.521z"/></g>'
  }
})