/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'lock': {
    width: 15,
    height: 20,
    viewBox: '0 0 15 20',
    data: '<path pid="0" d="M2.307 7.993h1.13V4.952c0-6.586 8.27-6.586 8.27 0v3.073h-1.206V4.952c0-5.002-5.857-5.002-5.857 0v3.041h8.194c1.182 0 2.162.955 2.162 2.162v6.836c0 1.207-.98 2.162-2.162 2.162H2.307a2.146 2.146 0 01-2.162-2.162v-6.836c0-1.207.955-2.162 2.162-2.162zm0 1.207h10.531c.528 0 .956.427.956.955v6.836a.955.955 0 01-.956.956H2.307a.955.955 0 01-.955-.956v-6.836c0-.528.427-.955.955-.955z" _fill="#000" fill-rule="evenodd"/>'
  }
})