/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'pencil': {
    width: 19,
    height: 19,
    viewBox: '0 0 19 19',
    data: '<path pid="0" d="M17.546 5.152L4.92 18.228 0 18.75l.503-5.097L11.48 2.282V2.28L13.128.575c.74-.767 2.03-.767 2.771 0l1.647 1.706c.368.38.575.897.574 1.436 0 .542-.204 1.051-.574 1.435zM1.78 16.904l2.415-.256L14.211 6.275 12.043 4.03 2.028 14.402l-.247 2.502zM14.513 1.629a.36.36 0 00-.26.112l-1.085 1.122 2.168 2.247 1.084-1.123a.392.392 0 000-.54L14.774 1.74a.36.36 0 00-.26-.112z" _fill="#D8D8D8" fill-rule="nonzero"/>'
  }
})