/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'reject': {
    width: 11,
    height: 11,
    viewBox: '0 0 11 11',
    data: '<defs><path pid="0" d="M33.422 29.988a.483.483 0 01-.681-.01l-2.647-2.646-2.573 2.573a.483.483 0 01-.681-.01l-1.612-1.61a.483.483 0 01-.01-.682l2.574-2.573-2.647-2.647a.483.483 0 01-.01-.68l1.567-1.567a.483.483 0 01.681.01l2.647 2.646 2.573-2.573a.483.483 0 01.681.01l1.612 1.61c.19.19.194.497.01.682l-2.574 2.573 2.647 2.647c.19.19.194.496.01.68l-1.567 1.567z" id="svgicon_reject_a"/></defs><use _fill="#FFF" fill-rule="nonzero" xlink:href="#svgicon_reject_a" transform="translate(-25 -20)"/>'
  }
})