/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'datasource': {
    width: 25,
    height: 22,
    viewBox: '0 0 25 22',
    data: '<path pid="0" d="M15.868 0l-3.683 4.49L8.868 0 0 10.814 8.846 21.6l3.511-4.254 3.488 4.254 8.436-10.217L15.868 0zm-2.862 16.56l4.275-5.176-4.482-6.064 3.028-3.693 7.185 9.724-7.163 8.676-2.843-3.466zM1.293 10.815l7.533-9.187 2.706 3.66L7 10.815l4.712 5.745-2.863 3.468-7.556-9.213zm7 0l3.852-4.697 3.867 5.234-3.651 4.422-4.068-4.96z" _fill="#0D0D0D" fill-rule="evenodd"/>'
  }
})