/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'info': {
    width: 16,
    height: 16,
    viewBox: '0 0 49.65 49.65',
    data: '<g data-name="Capa 2"><path pid="0" d="M24.82 0a24.83 24.83 0 1024.83 24.82A24.82 24.82 0 0024.82 0zm3.11 37.24a3.11 3.11 0 01-6.21 0V21.72a3.11 3.11 0 016.21 0zm-3.11-21.72a3.11 3.11 0 113.11-3.11 3.1 3.1 0 01-3.11 3.11z" _fill="#8176d4" data-name="Capa 1"/></g>'
  }
})