/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'check2': {
    width: 12,
    height: 9,
    viewBox: '0 0 12 9',
    data: '<path pid="0" d="M10.633 0c-.345.01-.673.142-.914.37-2.073 1.913-3.671 3.516-5.57 5.307L2.217 4.173a1.416 1.416 0 00-1.326-.246c-.457.147-.792.51-.873.95-.081.44.103.885.482 1.163l2.88 2.242c.527.411 1.31.38 1.799-.07 2.39-2.204 4.087-3.962 6.422-6.116.39-.352.506-.889.292-1.352-.214-.463-.715-.758-1.261-.743z" _fill="#FFF" fill-rule="nonzero"/>'
  }
})