/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'delete': {
    width: 17,
    height: 18,
    viewBox: '0 0 17 18',
    data: '<g _fill="#4A4A4A" fill-rule="nonzero"><path pid="0" d="M16.132 2.86h-4.876V1.678C11.256.862 10.595.2 9.78.2H6.97c-.816 0-1.477.662-1.477 1.478V2.86H.618a.493.493 0 000 .985h1.748v11.23a2.463 2.463 0 002.463 2.462h7.092a2.463 2.463 0 002.463-2.462V3.845h1.748a.493.493 0 000-.985zM6.48 1.678c0-.272.22-.493.492-.493H9.78c.272 0 .492.22.492.493V2.86H6.48V1.678zm6.895 13.421c0 .816-.661 1.478-1.477 1.478H4.829a1.478 1.478 0 01-1.478-1.478V3.845H13.4l-.025 11.254z"/><path pid="1" d="M10.375 14.5a.5.5 0 00.5-.5V6.525a.5.5 0 10-1 0V14a.5.5 0 00.5.5zM6.625 14.5a.5.5 0 00.5-.5V6.525a.5.5 0 00-1 0V14a.5.5 0 00.5.5z"/></g>'
  }
})