/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'edit': {
    width: 16,
    height: 16,
    viewBox: '0 0 16 16',
    data: '<defs><path pid="0" d="M29.52 17l-2.753 2.751.142-.141 3.481 3.481 2.61-2.61L29.52 17zM17 29.519V33h3.481l9.91-9.909-1.306 1.305-3.481-3.48L17 29.518z" id="svgicon_edit_a"/></defs><use _fill="#FFF" xlink:href="#svgicon_edit_a" transform="translate(-17 -17)" fill-rule="evenodd"/>'
  }
})