/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'arrow-top': {
    width: 17,
    height: 24,
    viewBox: '0 0 17 24',
    data: '<g _fill="#000" fill-rule="nonzero"><path pid="0" d="M9 1.028v21.944c0 .167-.3.278-.75.278s-.75-.111-.75-.278V1.028c0-.167.3-.278.75-.278s.75.111.75.278z"/><path pid="1" d="M9 .75a.68.68 0 01-.225.525l-7.5 7.5c-.3.3-.75.3-1.05 0-.3-.3-.3-.75 0-1.05l7.5-7.5c.3-.3.75-.3 1.05 0 .15.15.225.3.225.525z"/><path pid="2" d="M16.5 8.25a.68.68 0 01-.225.525c-.3.3-.75.3-1.05 0l-7.5-7.5c-.3-.3-.3-.75 0-1.05.3-.3.75-.3 1.05 0l7.5 7.5a.81.81 0 01.225.525z"/></g>'
  }
})