/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'export': {
    width: 15,
    height: 14,
    viewBox: '0 0 15 14',
    data: '<path pid="0" d="M7.928.22l3.63 3.553c.585.567-.294 1.418-.88.85l-2.445-2.37c-.12-.117-.218-.077-.218.09v6.542a.612.612 0 01-.622.603.61.61 0 01-.62-.603V2.342c0-.165-.098-.206-.219-.09L4.108 4.624c-.586.567-1.464-.284-.878-.85L6.835.223A.782.782 0 017.928.22zm5.067 12.277V10.39c0-.332.279-.601.623-.601.343 0 .622.27.622.601V13.1c0 .5-.42.901-.936.901H1.483c-.515 0-.936-.405-.936-.901V10.39c0-.332.278-.601.622-.601s.623.27.623.601v2.107c0 .163.139.3.31.3h10.582c.17 0 .311-.135.311-.3z" _fill="#2BABA6" fill-rule="nonzero"/>'
  }
})