/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
require('./annotate-view')
require('./arrow-bottom')
require('./arrow-top')
require('./back')
require('./brand-isotipo')
require('./brand-recognai')
require('./check')
require('./check2')
require('./check3')
require('./chev-bottom')
require('./chev-left')
require('./chev-right')
require('./chev-top')
require('./clasifier')
require('./clip')
require('./clock')
require('./copy-url')
require('./copy')
require('./create-datasource')
require('./cross')
require('./datasets-empty')
require('./datasource')
require('./datasources')
require('./delete')
require('./double-chev')
require('./drop-down')
require('./drop-up')
require('./dropbox')
require('./edit')
require('./editable')
require('./error')
require('./explore-view')
require('./export')
require('./external-link')
require('./external')
require('./feather')
require('./filter')
require('./forbidden')
require('./go-back')
require('./help')
require('./hipo')
require('./ignore')
require('./info')
require('./kebab-menu-h')
require('./kebab-menu-v')
require('./list')
require('./lock')
require('./logout')
require('./metrics')
require('./models')
require('./pagination-end')
require('./password-hide')
require('./password-show')
require('./pencil')
require('./plus')
require('./predicted-ko')
require('./predicted-ok')
require('./prev')
require('./progress')
require('./refresh')
require('./reject')
require('./search')
require('./sort')
require('./unlock')
require('./zoomin')
require('./zoomout')
