/** css/custom.css **/
@import url('https://fonts.googleapis.com/css2?family=Open+Sans:wght@300;400;600;700&display=swap');
:root {
  --sidebarWidth: 360px;
  --primaryColor: #0508D9;
  --sidebarBackground: #FFFFFF;
  --sidebarBackgroundDark: #fcfcfc;
  --sidebarBackgroundDarker: #F2F3F7;
  --sidebarColor: #484848;
  --sidebarTitleColor: #404040;;
  --white: #ffffff;
  --line: #e9eaed;
}

body {
  font-family: "Open Sans" ,"Helvetica Neue", "Arial", "sans-serif";
  color: #484848;
}
html {
  font-size: 16px;
}
h1 {
  font-size: 36px;
}
h2 {
  font-size: 28px;
}
#search-results {
  margin-top: 4em;
}
.document {
  margin-top: -2em;
}
.document .section h1 {
  margin-top: 2em;
}
.document .section h2 {
  margin-top: 1.5em;
}
.document .section h3 {
  margin-top: 2em;
}
.rst-footer-buttons {
  margin-top: 4em;
}
.wy-menu-vertical, .wy-menu-vertical p, .wy-menu-vertical a {
  font-size: 14px;
}
p {
  font-size: 16px;
  line-height: 1.8em;
}
.wy-breadcrumbs li:firt-child {
  color: var(--line);
}
.wy-breadcrumbs li:not(.wy-breadcrumbs-aside) a {
  color: var(--sidebarColor);
}
.wy-nav-side {
  background: var(--sidebarBackground);
  border-right: 1px solid var(--line);
}
.wy-menu-vertical a {
  padding-top: 0.65em !important;
  padding-bottom: 0.65em !important;
}
.wy-side-nav-search {
  background: none
}
.wy-nav-content {
  max-width: 1000px;
}
.icon-home:before {
  content: "Home";
  font-family: "Open Sans" ,"Helvetica Neue", "Arial", "sans-serif";
  -webkit-font-smoothing: auto;
}
.wy-breadcrumbs + hr {
  display: none;
}
.fa-plus-square-o:before, .wy-menu-vertical li span.toctree-expand:before,
.fa-minus-square-o:before, .wy-menu-vertical li.current>a span.toctree-expand:before, .wy-menu-vertical li.on a span.toctree-expand:before {
  content: none;
}
.wy-nav-content-wrap {
  background: #ffffff
}
.rst-content .toctree-wrapper>p.caption, h1, h2, h3, h4, h5, h6, legend {
  font-family: 'Futura Medium';
  font-weight: normal;
}
.wy-menu-vertical li.toctree-l1.current>a {
  border: none;
}
.wy-menu-vertical li.current>a, .wy-menu-vertical li.on a {
  color: var(--sidebarColor);
}
.wy-menu-vertical a:hover {
  background: var(--sidebarBackgroundDark);
  color: var(--sidebarColor);
}
.wy-menu-vertical li.current {
  background: var(--sidebarBackgroundDark)
}
.wy-menu-vertical li.current a {
  border: none;
}
.wy-menu-vertical li.toctree-l2.current>a, .wy-menu-vertical li.toctree-l2.current li.toctree-l3>a {
  background: #d3d3d4
}
.wy-menu-vertical li.toctree-l2 a, .wy-menu-vertical li.toctree-l3 a, .wy-menu-vertical li.toctree-l4 a, .wy-menu-vertical li.toctree-l5 a, .wy-menu-vertical li.toctree-l6 a, .wy-menu-vertical li.toctree-l7 a, .wy-menu-vertical li.toctree-l8 a, .wy-menu-vertical li.toctree-l9 a, .wy-menu-vertical li.toctree-l10 a {
  color: var(--sidebarColor);
}
.wy-menu-vertical a, .wy-menu-vertical li.current>a span.toctree-expand, .wy-menu-vertical li.on a span.toctree-expand {
  color: var(--sidebarColor);
  font-size: 14px;
  line-height: 1.8em;
}
.rst-content code.literal, .rst-content tt.literal {
  color: #F2067A;
}
.wy-menu-vertical li.toctree-l2.current>a, .wy-menu-vertical li.toctree-l2.current li.toctree-l3>a {
  background: var(--sidebarBackgroundDarker)
}
.wy-menu-vertical li.current a:hover {
  background: var(--sidebarBackgroundDarker)
}
.wy-menu-vertical li.toctree-l1.current>a {
  background: var(--sidebarBackgroundDarker)
}
.wy-menu-vertical header, .wy-menu-vertical p.caption {
  color: #4C4EA3;
  font-size: 22px;
  font-family: 'Futura Medium';
  font-weight: normal;
  padding: 0 20px;
  text-transform: none;
}

a {
  color: var(--primaryColor)
}
a:hover {
  color: #0709ae
}
.btn-neutral {
  background: none !important;
  box-shadow: none !important;
}
.btn-neutral:hover {
  border: 1px solid rgba(0,0,0,.2);
  background: none !important;
}
.btn-neutral span {
  display: none !important;
}
.wy-side-nav-search input[type=text] {
  border-radius: 3px;
  border: 1px solid #e9eaed;
  padding: 0 1em 0 3em;
  outline: none;
  background: transparent;
  min-height: 40px;
  min-width: 200px;
  box-shadow: none;
  background: url('../images/search.svg') no-repeat 10px 12px;
  font-size: 14px;
}

.rst-content div[class^=highlight], .rst-content pre.literal-block {
  border: none;
  border-radius: 5px;
}
.wy-side-nav-search {
  text-align: left;
}
.wy-side-nav-search .wy-dropdown>a, .wy-side-nav-search>a {
  padding-left: 0;
}
.wy-side-nav-search img {
  max-width: 120px !important;
  padding-left: 0;
}
.wy-nav-top {
  background: var(--primaryColor)
}

.highlight { background-color: #333346; color: white; padding: 1em !important; }
.highlight .c { color: #82d6eb; font-style: italic } /* Comment */
.highlight .err { border: 1px solid #FF0000 } /* Error */
.highlight .k { color: #3ef070; font-weight: bold } /* Keyword */
.highlight .o { color: #dddcdc } /* Operator */
.highlight .cm { color: #82d6eb; font-style: italic } /* Comment.Multiline */
.highlight .cp { color: #3ef070 } /* Comment.Preproc */
.highlight .c1 { color: #82d6eb; font-style: italic } /* Comment.Single */
.highlight .cs { color: #82d6eb; background-color: #fff0f0 } /* Comment.Special */
.highlight .gd { color: rgb(242, 243, 167) } /* Generic.Deleted */
.highlight .ge { font-style: italic } /* Generic.Emph */
.highlight .gr { color: rgb(255, 222, 130) } /* Generic.Error */
.highlight .gh { color: #c1c1e2; font-weight: bold } /* Generic.Heading */
.highlight .gi { color: #3ef070 } /* Generic.Inserted */
.highlight .go { color: #808080 } /* Generic.Output */
.highlight .gp { color: #c65d09; font-weight: bold } /* Generic.Prompt */
.highlight .gs { font-weight: bold } /* Generic.Strong */
.highlight .gu { color: #e2a1e2; font-weight: bold } /* Generic.Subheading */
.highlight .gt { color: rgb(153, 181, 245) } /* Generic.Traceback */
.highlight .kc { color: #3ef070; font-weight: bold } /* Keyword.Constant */
.highlight .kd { color: #3ef070; font-weight: bold } /* Keyword.Declaration */
.highlight .kn { color: #3ef070; font-weight: bold } /* Keyword.Namespace */
.highlight .kp { color: #3ef070 } /* Keyword.Pseudo */
.highlight .kr { color: #3ef070; font-weight: bold } /* Keyword.Reserved */
.highlight .kt { color: #f79cc2 } /* Keyword.Type */
.highlight .m { color: #fbffc2 } /* Literal.Number */
.highlight .s { color: #a0c7ee } /* Literal.String */
.highlight .na { color: #a0c7ee } /* Name.Attribute */
.highlight .nb { color: #f089d5; } /* Name.Builtin */
.highlight .nc { color: #48c1f5; font-weight: bold } /* Name.Class */
.highlight .no { color: #60add5 } /* Name.Constant */
.highlight .nd { color: white; font-weight: bold } /* Name.Decorator */
.highlight .ni { color: #d55537; font-weight: bold } /* Name.Entity */
.highlight .ne { color: #3ef070 } /* Name.Exception */
.highlight .nf { color: #8aa7f0 } /* Name.Function */
.highlight .fm { color: rgb(193, 193, 247) }
.highlight .nl { color: #8aa7f0; font-weight: bold } /* Name.Label */
.highlight .nn { color: #48c1f5; font-weight: bold } /* Name.Namespace */
.highlight .nt { color: #8ba2d3; font-weight: bold } /* Name.Tag */
.highlight .nv { color: #d5abe0 } /* Name.Variable */
.highlight .ow { color: #3ef070; font-weight: bold } /* Operator.Word */
.highlight .w { color: #bbbbbb } /* Text.Whitespace */
.highlight .mf { color: #fbffc2 } /* Literal.Number.Float */
.highlight .mh { color: #fbffc2 } /* Literal.Number.Hex */
.highlight .mi { color: #fbffc2 } /* Literal.Number.Integer */
.highlight .mo { color: #fbffc2 } /* Literal.Number.Oct */
.highlight .sb { color: #a0c7ee } /* Literal.String.Backtick */
.highlight .sc { color: #a0c7ee } /* Literal.String.Char */
.highlight .sd { color: #a0c7ee; font-style: italic } /* Literal.String.Doc */
.highlight .s2 { color: #a0c7ee } /* Literal.String.Double */
.highlight .se { color: #a0c7ee; font-weight: bold } /* Literal.String.Escape */
.highlight .sh { color: #a0c7ee } /* Literal.String.Heredoc */
.highlight .si { color: #70a0d0; font-style: italic } /* Literal.String.Interpol */
.highlight .sx { color: #f8b884 } /* Literal.String.Other */
.highlight .sr { color: #4f9ff3 } /* Literal.String.Regex */
.highlight .s1 { color: #a0c7ee } /* Literal.String.Single */
.highlight .ss { color: #aff849 } /* Literal.String.Symbol */
.highlight .bp { color: #3ef070 } /* Name.Builtin.Pseudo */
.highlight .vc { color: #d5abe0 } /* Name.Variable.Class */
.highlight .vg { color: #d5abe0 } /* Name.Variable.Global */
.highlight .vi { color: #d5abe0 } /* Name.Variable.Instance */
.highlight .il { color: #fbffc2 } 

@font-face {
  font-family: 'Futura Medium';
  font-style: normal;
  font-weight: normal;
  src: local('Futura'), url('fonts/futura-medium.woff2') format('woff2');
}

@font-face {
  font-family: 'Futura Medium Condensed';
  font-style: normal;
  font-weight: normal;
  src: local('Futura Medium Condensed'), url('fonts/futura-medium-condensed.woff2') format('woff2');
}
  
  
  @font-face {
  font-family: 'Futura Bold';
  font-style: normal;
  font-weight: normal;
  src: local('Futura Bold'), url('fonts/futura-bold.woff2') format('woff2');
  }
  
  
  @font-face {
  font-family: 'Futura Light';
  font-style: normal;
  font-weight: normal;
  src: local('Futura Light'), url('fonts/futura-light.woff2') format('woff2');
}

@media screen and (min-width: 1100px) {
  .wy-nav-content {
    padding: 2em 4.5em;
    background: #ffffff
  }
  .wy-nav-content-wrap {
    margin-left: var(--sidebarWidth);
  }
  .wy-menu-vertical p.caption {
    padding-left: 26px;
  }
  .wy-menu-vertical {
    width: var(--sidebarWidth);
  }
  .wy-nav-side {
    width: var(--sidebarWidth);
  }
  .wy-side-scroll {
    width: var(--sidebarWidth);
  }
  .wy-side-nav-search,
  .wy-menu-vertical li.toctree-l1.current>a,
  .wy-menu-vertical a {
    padding-left: 2em;
    padding-right: 2em;
  }
  .wy-menu-vertical li.toctree-l2.current>a {
    padding-left: 34px;
    padding-right: 2em;  
  };
  .wy-menu-vertical li.toctree-l2.current li.toctree-l3>a, 
  .wy-menu-vertical li.toctree-l3.current>a {
    padding-left: 4em;
    padding-right: 2em;    
  }
}
/* Change the white to any color */
input:-webkit-autofill,
input:-webkit-autofill:hover, 
input:-webkit-autofill:focus, 
input:-webkit-autofill:active
{
 -webkit-box-shadow: 0 0 0 30px white inset !important;
}

.rst-content .highlighted {
  background: #c5c6e0;
  box-shadow: 0 0 0 2px #c5c6e0;
}