/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'hipo': {
    width: 17,
    height: 41,
    viewBox: '0 0 17 41',
    data: '<path pid="0" d="M6.279 0L17 6.962l-1.663 1.839-6.542-1.839c3.107 2.515 4.568 5.867 4.382 10.056-.185 4.189-1.981 8.398-5.388 12.628l-1.348 3.75-.605 3.956L8.1 39.16l1.97-.646.484-1.64-1.15-.89-.904.651.718.572.558-.572v1.004l-.558.417-1.46-.85v-1.223l1.646-1.14 2.394 1.421v2.25l-1.244 1.778L8.1 41l-2.264-.491-2.101-2.001-.764-1.156-.08-5.28 2.114-8.498-2.667 1.88L0 23.715v-8.102l2.97 1.405-2.5-11.5L6.278 0z" _fill="#FBD8AA" fill-rule="evenodd"/>'
  }
})