/*
 * coding=utf-8
 * Copyright 2021-present, the Recognai S.L. team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* eslint-disable */
var icon = require('vue-svgicon')
icon.register({
  'chev-top': {
    width: 7,
    height: 5,
    viewBox: '0 0 7 5',
    data: '<path pid="0" d="M3.374 2.396l-2.21 2.276L0 3.474l2.21-2.276L3.375 0l1.164 1.198 2.21 2.276-1.163 1.198z" _fill="#4C4EA3" fill-rule="evenodd"/>'
  }
})